@@ -49,7 +49,6 @@ import org.apache.flink.streaming.runtime.io.MultipleFuturesAvailabilityHelper;
 import org.apache.flink.util.Preconditions;
 import org.apache.flink.util.UserCodeClassLoader;
 
-import com.google.common.collect.ArrayListMultimap;
 import org.apache.kafka.clients.consumer.ConsumerRecord;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -92,6 +91,7 @@ public class DynamicKafkaSourceReader<T> implements SourceReader<T, DynamicKafka
     private final List<DynamicKafkaSourceSplit> pendingSplits;
 
     private MultipleFuturesAvailabilityHelper availabilityHelper;
+    private int availabilityHelperSize;
     private boolean isActivelyConsumingSplits;
     private boolean isNoMoreSplits;
     private AtomicBoolean restartingReaders;
@@ -111,7 +111,8 @@ public class DynamicKafkaSourceReader<T> implements SourceReader<T, DynamicKafka
                         .addGroup(KafkaClusterMetricGroup.DYNAMIC_KAFKA_SOURCE_METRIC_GROUP);
         this.kafkaClusterMetricGroupManager = new KafkaClusterMetricGroupManager();
         this.pendingSplits = new ArrayList<>();
-        this.availabilityHelper = new MultipleFuturesAvailabilityHelper(0);
+        this.availabilityHelper =
+                new MultipleFuturesAvailabilityHelper(this.availabilityHelperSize = 0);
         this.isNoMoreSplits = false;
         this.isActivelyConsumingSplits = false;
         this.restartingReaders = new AtomicBoolean();
@@ -185,10 +186,11 @@ public class DynamicKafkaSourceReader<T> implements SourceReader<T, DynamicKafka
             return;
         }
 
-        ArrayListMultimap<String, KafkaPartitionSplit> clusterSplitsMap =
-                ArrayListMultimap.create();
+        Map<String, List<KafkaPartitionSplit>> clusterSplitsMap = new HashMap<>();
         for (DynamicKafkaSourceSplit split : splits) {
-            clusterSplitsMap.put(split.getKafkaClusterId(), split);
+            clusterSplitsMap
+                    .computeIfAbsent(split.getKafkaClusterId(), unused -> new ArrayList<>())
+                    .add(split);
         }
 
         Set<String> kafkaClusterIds = clusterSplitsMap.keySet();
@@ -472,7 +474,9 @@ public class DynamicKafkaSourceReader<T> implements SourceReader<T, DynamicKafka
      */
     private void completeAndResetAvailabilityHelper() {
         CompletableFuture<?> cachedPreviousFuture = availabilityHelper.getAvailableFuture();
-        availabilityHelper = new MultipleFuturesAvailabilityHelper(clusterReaderMap.size());
+        availabilityHelper =
+                new MultipleFuturesAvailabilityHelper(
+                        this.availabilityHelperSize = clusterReaderMap.size());
         syncAvailabilityHelperWithReaders();
 
         // We cannot immediately complete the previous future here. We must complete it only when
@@ -538,8 +542,8 @@ public class DynamicKafkaSourceReader<T> implements SourceReader<T, DynamicKafka
     }
 
     @VisibleForTesting
-    public MultipleFuturesAvailabilityHelper getAvailabilityHelper() {
-        return availabilityHelper;
+    public int getAvailabilityHelperSize() {
+        return availabilityHelperSize;
     }
 
     @VisibleForTesting
