@@ -21,13 +21,21 @@ import org.apache.flink.annotation.Internal;
 import org.apache.flink.annotation.PublicEvolving;
 import org.apache.flink.annotation.VisibleForTesting;
 import org.apache.flink.api.connector.sink2.Committer;
+import org.apache.flink.api.connector.sink2.CommitterInitContext;
+import org.apache.flink.api.connector.sink2.WriterInitContext;
+import org.apache.flink.api.dag.Transformation;
 import org.apache.flink.connector.base.DeliveryGuarantee;
 import org.apache.flink.connector.kafka.lineage.KafkaDatasetFacet;
 import org.apache.flink.connector.kafka.lineage.KafkaDatasetFacetProvider;
 import org.apache.flink.connector.kafka.lineage.LineageUtil;
 import org.apache.flink.connector.kafka.lineage.TypeDatasetFacet;
 import org.apache.flink.connector.kafka.lineage.TypeDatasetFacetProvider;
+import org.apache.flink.connector.kafka.sink.internal.FlinkKafkaInternalProducer;
 import org.apache.flink.core.io.SimpleVersionedSerializer;
+import org.apache.flink.streaming.api.connector.sink2.CommittableMessage;
+import org.apache.flink.streaming.api.connector.sink2.CommittableMessageTypeInfo;
+import org.apache.flink.streaming.api.connector.sink2.SupportsPostCommitTopology;
+import org.apache.flink.streaming.api.datastream.DataStream;
 import org.apache.flink.streaming.api.lineage.LineageVertex;
 import org.apache.flink.streaming.api.lineage.LineageVertexProvider;
 
@@ -66,23 +74,27 @@ import java.util.Properties;
 @PublicEvolving
 public class KafkaSink<IN>
         implements LineageVertexProvider,
-                TwoPhaseCommittingStatefulSink<IN, KafkaWriterState, KafkaCommittable> {
+                TwoPhaseCommittingStatefulSink<IN, KafkaWriterState, KafkaCommittable>,
+                SupportsPostCommitTopology<KafkaCommittable> {
     private static final Logger LOG = LoggerFactory.getLogger(KafkaSink.class);
     private final DeliveryGuarantee deliveryGuarantee;
 
     private final KafkaRecordSerializationSchema<IN> recordSerializer;
     private final Properties kafkaProducerConfig;
     private final String transactionalIdPrefix;
+    private final TransactionNamingStrategy transactionNamingStrategy;
 
     KafkaSink(
             DeliveryGuarantee deliveryGuarantee,
             Properties kafkaProducerConfig,
             String transactionalIdPrefix,
-            KafkaRecordSerializationSchema<IN> recordSerializer) {
+            KafkaRecordSerializationSchema<IN> recordSerializer,
+            TransactionNamingStrategy transactionNamingStrategy) {
         this.deliveryGuarantee = deliveryGuarantee;
         this.kafkaProducerConfig = kafkaProducerConfig;
         this.transactionalIdPrefix = transactionalIdPrefix;
         this.recordSerializer = recordSerializer;
+        this.transactionNamingStrategy = transactionNamingStrategy;
     }
 
     /**
@@ -97,8 +109,14 @@ public class KafkaSink<IN>
 
     @Internal
     @Override
-    public Committer<KafkaCommittable> createCommitter() throws IOException {
-        return new KafkaCommitter(kafkaProducerConfig);
+    public Committer<KafkaCommittable> createCommitter(CommitterInitContext context) {
+        return new KafkaCommitter(
+                kafkaProducerConfig,
+                transactionalIdPrefix,
+                context.getTaskInfo().getIndexOfThisSubtask(),
+                context.getTaskInfo().getAttemptNumber(),
+                transactionNamingStrategy == TransactionNamingStrategy.POOLING,
+                FlinkKafkaInternalProducer::new);
     }
 
     @Internal
@@ -109,29 +127,38 @@ public class KafkaSink<IN>
 
     @Internal
     @Override
-    public KafkaWriter<IN> createWriter(InitContext context) throws IOException {
-        return new KafkaWriter<IN>(
-                deliveryGuarantee,
-                kafkaProducerConfig,
-                transactionalIdPrefix,
-                context,
-                recordSerializer,
-                context.asSerializationSchemaInitializationContext(),
-                Collections.emptyList());
+    public KafkaWriter<IN> createWriter(WriterInitContext context) throws IOException {
+        return restoreWriter(context, Collections.emptyList());
     }
 
     @Internal
     @Override
     public KafkaWriter<IN> restoreWriter(
-            InitContext context, Collection<KafkaWriterState> recoveredState) throws IOException {
-        return new KafkaWriter<>(
-                deliveryGuarantee,
-                kafkaProducerConfig,
-                transactionalIdPrefix,
-                context,
-                recordSerializer,
-                context.asSerializationSchemaInitializationContext(),
-                recoveredState);
+            WriterInitContext context, Collection<KafkaWriterState> recoveredState) {
+        KafkaWriter<IN> writer;
+        if (deliveryGuarantee == DeliveryGuarantee.EXACTLY_ONCE) {
+            writer =
+                    new ExactlyOnceKafkaWriter<>(
+                            deliveryGuarantee,
+                            kafkaProducerConfig,
+                            transactionalIdPrefix,
+                            context,
+                            recordSerializer,
+                            context.asSerializationSchemaInitializationContext(),
+                            transactionNamingStrategy.getAbortImpl(),
+                            transactionNamingStrategy.getImpl(),
+                            recoveredState);
+        } else {
+            writer =
+                    new KafkaWriter<>(
+                            deliveryGuarantee,
+                            kafkaProducerConfig,
+                            context,
+                            recordSerializer,
+                            context.asSerializationSchemaInitializationContext());
+        }
+        writer.initialize();
+        return writer;
     }
 
     @Internal
@@ -140,6 +167,22 @@ public class KafkaSink<IN>
         return new KafkaWriterStateSerializer();
     }
 
+    @Override
+    public void addPostCommitTopology(DataStream<CommittableMessage<KafkaCommittable>> committer) {
+        // this is a somewhat hacky way to ensure that the committer and writer are co-located
+        if (deliveryGuarantee == DeliveryGuarantee.EXACTLY_ONCE && transactionalIdPrefix != null) {
+            Transformation<?> transformation = committer.getTransformation();
+            // all sink transformations output CommittableMessage, so we can safely traverse the
+            // chain; custom colocation key is set before and should be preserved
+            while (transformation.getOutputType() instanceof CommittableMessageTypeInfo
+                    && transformation.getCoLocationGroupKey() == null) {
+                // colocate by transactionalIdPrefix, which should be unique
+                transformation.setCoLocationGroupKey(transactionalIdPrefix);
+                transformation = transformation.getInputs().get(0);
+            }
+        }
+    }
+
     @VisibleForTesting
     protected Properties getKafkaProducerConfig() {
         return kafkaProducerConfig;
