@@ -26,7 +26,9 @@ import org.apache.flink.api.dag.Transformation;
 import org.apache.flink.configuration.ConfigOptions;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.connector.base.DeliveryGuarantee;
+import org.apache.flink.connector.kafka.sink.KafkaPartitioner;
 import org.apache.flink.connector.kafka.sink.KafkaSink;
+import org.apache.flink.connector.kafka.sink.TransactionNamingStrategy;
 import org.apache.flink.connector.kafka.source.KafkaSource;
 import org.apache.flink.connector.kafka.source.KafkaSourceOptions;
 import org.apache.flink.connector.kafka.source.KafkaSourceTestUtils;
@@ -42,9 +44,7 @@ import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.streaming.api.transformations.SourceTransformation;
 import org.apache.flink.streaming.connectors.kafka.config.BoundedMode;
 import org.apache.flink.streaming.connectors.kafka.config.StartupMode;
-import org.apache.flink.streaming.connectors.kafka.internals.KafkaTopicPartition;
 import org.apache.flink.streaming.connectors.kafka.partitioner.FlinkFixedPartitioner;
-import org.apache.flink.streaming.connectors.kafka.partitioner.FlinkKafkaPartitioner;
 import org.apache.flink.streaming.connectors.kafka.table.KafkaConnectorOptions.ScanStartupMode;
 import org.apache.flink.streaming.connectors.kafka.testutils.MockPartitionOffsetsRetriever;
 import org.apache.flink.table.api.DataTypes;
@@ -101,6 +101,7 @@ import static org.apache.flink.core.testutils.FlinkAssertions.anyCauseMatches;
 import static org.apache.flink.streaming.connectors.kafka.table.KafkaConnectorOptionsUtil.AVRO_CONFLUENT;
 import static org.apache.flink.streaming.connectors.kafka.table.KafkaConnectorOptionsUtil.DEBEZIUM_AVRO_CONFLUENT;
 import static org.apache.flink.streaming.connectors.kafka.table.KafkaConnectorOptionsUtil.PROPERTIES_PREFIX;
+import static org.apache.flink.table.factories.FactoryUtil.SOURCE_PARALLELISM;
 import static org.apache.flink.table.factories.utils.FactoryMocks.createTableSink;
 import static org.apache.flink.table.factories.utils.FactoryMocks.createTableSource;
 import static org.assertj.core.api.Assertions.assertThat;
@@ -110,7 +111,6 @@ import static org.assertj.core.api.Assertions.assertThatThrownBy;
 /** Tests for {@link KafkaDynamicTableFactory}. */
 @ExtendWith(TestLoggerExtension.class)
 public class KafkaDynamicTableFactoryTest {
-
     private static final String TOPIC = "myTopic";
     private static final String TOPICS = "myTopic-1;myTopic-2;myTopic-3";
     private static final String TOPIC_REGEX = "myTopic-\\d+";
@@ -191,9 +191,9 @@ public class KafkaDynamicTableFactoryTest {
         final DynamicTableSource actualSource = createTableSource(SCHEMA, getBasicSourceOptions());
         final KafkaDynamicSource actualKafkaSource = (KafkaDynamicSource) actualSource;
 
-        final Map<KafkaTopicPartition, Long> specificOffsets = new HashMap<>();
-        specificOffsets.put(new KafkaTopicPartition(TOPIC, PARTITION_0), OFFSET_0);
-        specificOffsets.put(new KafkaTopicPartition(TOPIC, PARTITION_1), OFFSET_1);
+        final Map<TopicPartition, Long> specificOffsets = new HashMap<>();
+        specificOffsets.put(new TopicPartition(TOPIC, PARTITION_0), OFFSET_0);
+        specificOffsets.put(new TopicPartition(TOPIC, PARTITION_1), OFFSET_1);
 
         final DecodingFormat<DeserializationSchema<RowData>> valueDecodingFormat =
                 new DecodingFormatMock(",", true);
@@ -212,7 +212,8 @@ public class KafkaDynamicTableFactoryTest {
                         KAFKA_SOURCE_PROPERTIES,
                         StartupMode.SPECIFIC_OFFSETS,
                         specificOffsets,
-                        0);
+                        0,
+                        null);
         assertThat(actualKafkaSource).isEqualTo(expectedKafkaSource);
 
         ScanTableSource.ScanRuntimeProvider provider =
@@ -220,6 +221,48 @@ public class KafkaDynamicTableFactoryTest {
         assertKafkaSource(provider);
     }
 
+    @Test
+    public void testTableSourceWithParallelism() {
+        final Map<String, String> modifiedOptions =
+                getModifiedOptions(
+                        getBasicSourceOptions(),
+                        options -> options.put(SOURCE_PARALLELISM.key(), "100"));
+        final DynamicTableSource actualSource = createTableSource(SCHEMA, modifiedOptions);
+        final KafkaDynamicSource actualKafkaSource = (KafkaDynamicSource) actualSource;
+
+        final Map<TopicPartition, Long> specificOffsets = new HashMap<>();
+        specificOffsets.put(new TopicPartition(TOPIC, PARTITION_0), OFFSET_0);
+        specificOffsets.put(new TopicPartition(TOPIC, PARTITION_1), OFFSET_1);
+
+        final DecodingFormat<DeserializationSchema<RowData>> valueDecodingFormat =
+                new DecodingFormatMock(",", true);
+
+        // Test scan source equals
+        final KafkaDynamicSource expectedKafkaSource =
+                createExpectedScanSource(
+                        SCHEMA_DATA_TYPE,
+                        null,
+                        valueDecodingFormat,
+                        new int[0],
+                        new int[] {0, 1, 2},
+                        null,
+                        Collections.singletonList(TOPIC),
+                        null,
+                        KAFKA_SOURCE_PROPERTIES,
+                        StartupMode.SPECIFIC_OFFSETS,
+                        specificOffsets,
+                        0,
+                        100);
+        assertThat(actualKafkaSource).isEqualTo(expectedKafkaSource);
+
+        ScanTableSource.ScanRuntimeProvider provider =
+                actualKafkaSource.getScanRuntimeProvider(ScanRuntimeProviderContext.INSTANCE);
+        assertThat(provider).isInstanceOf(DataStreamScanProvider.class);
+        final DataStreamScanProvider sourceProvider = (DataStreamScanProvider) provider;
+        assertThat(sourceProvider.getParallelism()).isPresent();
+        assertThat(sourceProvider.getParallelism()).hasValue(100);
+    }
+
     @Test
     public void testTableSourceWithPattern() {
         final Map<String, String> modifiedOptions =
@@ -235,7 +278,7 @@ public class KafkaDynamicTableFactoryTest {
                         });
         final DynamicTableSource actualSource = createTableSource(SCHEMA, modifiedOptions);
 
-        final Map<KafkaTopicPartition, Long> specificOffsets = new HashMap<>();
+        final Map<TopicPartition, Long> specificOffsets = new HashMap<>();
 
         DecodingFormat<DeserializationSchema<RowData>> valueDecodingFormat =
                 new DecodingFormatMock(",", true);
@@ -254,7 +297,8 @@ public class KafkaDynamicTableFactoryTest {
                         KAFKA_SOURCE_PROPERTIES,
                         StartupMode.EARLIEST,
                         specificOffsets,
-                        0);
+                        0,
+                        null);
         final KafkaDynamicSource actualKafkaSource = (KafkaDynamicSource) actualSource;
         assertThat(actualKafkaSource).isEqualTo(expectedKafkaSource);
 
@@ -295,7 +339,8 @@ public class KafkaDynamicTableFactoryTest {
                         KAFKA_FINAL_SOURCE_PROPERTIES,
                         StartupMode.GROUP_OFFSETS,
                         Collections.emptyMap(),
-                        0);
+                        0,
+                        null);
 
         assertThat(actualSource).isEqualTo(expectedKafkaSource);
     }
@@ -346,7 +391,8 @@ public class KafkaDynamicTableFactoryTest {
                         KAFKA_FINAL_SOURCE_PROPERTIES,
                         StartupMode.GROUP_OFFSETS,
                         Collections.emptyMap(),
-                        0);
+                        0,
+                        null);
         expectedKafkaSource.producedDataType = SCHEMA_WITH_METADATA.toSourceRowDataType();
         expectedKafkaSource.metadataKeys = Collections.singletonList("timestamp");
 
@@ -606,7 +652,8 @@ public class KafkaDynamicTableFactoryTest {
                         new FlinkFixedPartitioner<>(),
                         DeliveryGuarantee.EXACTLY_ONCE,
                         null,
-                        "kafka-sink");
+                        "kafka-sink",
+                        TransactionNamingStrategy.DEFAULT);
         assertThat(actualSink).isEqualTo(expectedSink);
 
         // Test kafka producer.
@@ -651,7 +698,41 @@ public class KafkaDynamicTableFactoryTest {
                             new FlinkFixedPartitioner<>(),
                             DeliveryGuarantee.valueOf(semantic.toUpperCase().replace("-", "_")),
                             null,
-                            "kafka-sink");
+                            "kafka-sink",
+                            TransactionNamingStrategy.DEFAULT);
+            assertThat(actualSink).isEqualTo(expectedSink);
+        }
+    }
+
+    @Test
+    public void testTableSinkStrategyTranslation() {
+        for (TransactionNamingStrategy namingStrategy : TransactionNamingStrategy.values()) {
+            final EncodingFormat<SerializationSchema<RowData>> valueEncodingFormat =
+                    new EncodingFormatMock(",");
+            final Map<String, String> modifiedOptions =
+                    getModifiedOptions(
+                            getBasicSinkOptions(),
+                            options -> {
+                                options.put(
+                                        "sink.transaction-naming-strategy", namingStrategy.name());
+                            });
+            final DynamicTableSink actualSink = createTableSink(SCHEMA, modifiedOptions);
+            final DynamicTableSink expectedSink =
+                    createExpectedSink(
+                            SCHEMA_DATA_TYPE,
+                            null,
+                            valueEncodingFormat,
+                            new int[0],
+                            new int[] {0, 1, 2},
+                            null,
+                            Collections.singletonList(TOPIC),
+                            null,
+                            KAFKA_SINK_PROPERTIES,
+                            new FlinkFixedPartitioner<>(),
+                            DeliveryGuarantee.EXACTLY_ONCE,
+                            null,
+                            "kafka-sink",
+                            namingStrategy);
             assertThat(actualSink).isEqualTo(expectedSink);
         }
     }
@@ -695,7 +776,8 @@ public class KafkaDynamicTableFactoryTest {
                         new FlinkFixedPartitioner<>(),
                         DeliveryGuarantee.EXACTLY_ONCE,
                         null,
-                        "kafka-sink");
+                        "kafka-sink",
+                        TransactionNamingStrategy.DEFAULT);
 
         assertThat(actualSink).isEqualTo(expectedSink);
     }
@@ -724,7 +806,8 @@ public class KafkaDynamicTableFactoryTest {
                         new FlinkFixedPartitioner<>(),
                         DeliveryGuarantee.EXACTLY_ONCE,
                         100,
-                        "kafka-sink");
+                        "kafka-sink",
+                        TransactionNamingStrategy.DEFAULT);
         assertThat(actualSink).isEqualTo(expectedSink);
 
         final DynamicTableSink.SinkRuntimeProvider provider =
@@ -836,7 +919,8 @@ public class KafkaDynamicTableFactoryTest {
                         new FlinkFixedPartitioner<>(),
                         DeliveryGuarantee.EXACTLY_ONCE,
                         null,
-                        "kafka-sink");
+                        "kafka-sink",
+                        TransactionNamingStrategy.DEFAULT);
         assertThat(actualSink).isEqualTo(expectedSink);
         final KafkaDynamicSink actualKafkaSink = (KafkaDynamicSink) actualSink;
         assertThat(actualKafkaSink.listWritableMetadata())
@@ -874,7 +958,8 @@ public class KafkaDynamicTableFactoryTest {
                         new FlinkFixedPartitioner<>(),
                         DeliveryGuarantee.EXACTLY_ONCE,
                         null,
-                        "kafka-sink");
+                        "kafka-sink",
+                        TransactionNamingStrategy.DEFAULT);
         assertThat(actualSink).isEqualTo(expectedSink);
         final KafkaDynamicSink actualKafkaSink = (KafkaDynamicSink) actualSink;
         assertThat(actualKafkaSink.listWritableMetadata())
@@ -1169,9 +1254,9 @@ public class KafkaDynamicTableFactoryTest {
         props.putAll(KAFKA_SOURCE_PROPERTIES);
         // The default partition discovery interval is 5 minutes
         props.setProperty("partition.discovery.interval.ms", "300000");
-        final Map<KafkaTopicPartition, Long> specificOffsets = new HashMap<>();
-        specificOffsets.put(new KafkaTopicPartition(TOPIC, PARTITION_0), OFFSET_0);
-        specificOffsets.put(new KafkaTopicPartition(TOPIC, PARTITION_1), OFFSET_1);
+        final Map<TopicPartition, Long> specificOffsets = new HashMap<>();
+        specificOffsets.put(new TopicPartition(TOPIC, PARTITION_0), OFFSET_0);
+        specificOffsets.put(new TopicPartition(TOPIC, PARTITION_1), OFFSET_1);
         final DecodingFormat<DeserializationSchema<RowData>> valueDecodingFormat =
                 new DecodingFormatMock(",", true);
         // Test scan source equals
@@ -1188,7 +1273,8 @@ public class KafkaDynamicTableFactoryTest {
                         props,
                         StartupMode.SPECIFIC_OFFSETS,
                         specificOffsets,
-                        0);
+                        0,
+                        null);
         assertThat(actualSource).isEqualTo(expectedKafkaSource);
         ScanTableSource.ScanRuntimeProvider provider =
                 actualSource.getScanRuntimeProvider(ScanRuntimeProviderContext.INSTANCE);
@@ -1207,9 +1293,9 @@ public class KafkaDynamicTableFactoryTest {
         props.putAll(KAFKA_SOURCE_PROPERTIES);
         // Disable discovery if the partition discovery interval is 0 minutes
         props.setProperty("partition.discovery.interval.ms", "0");
-        final Map<KafkaTopicPartition, Long> specificOffsets = new HashMap<>();
-        specificOffsets.put(new KafkaTopicPartition(TOPIC, PARTITION_0), OFFSET_0);
-        specificOffsets.put(new KafkaTopicPartition(TOPIC, PARTITION_1), OFFSET_1);
+        final Map<TopicPartition, Long> specificOffsets = new HashMap<>();
+        specificOffsets.put(new TopicPartition(TOPIC, PARTITION_0), OFFSET_0);
+        specificOffsets.put(new TopicPartition(TOPIC, PARTITION_1), OFFSET_1);
         final DecodingFormat<DeserializationSchema<RowData>> valueDecodingFormat =
                 new DecodingFormatMock(",", true);
         // Test scan source equals
@@ -1226,7 +1312,8 @@ public class KafkaDynamicTableFactoryTest {
                         props,
                         StartupMode.SPECIFIC_OFFSETS,
                         specificOffsets,
-                        0);
+                        0,
+                        null);
         assertThat(actualSource).isEqualTo(expectedKafkaSource);
         ScanTableSource.ScanRuntimeProvider provider =
                 actualSource.getScanRuntimeProvider(ScanRuntimeProviderContext.INSTANCE);
@@ -1248,8 +1335,9 @@ public class KafkaDynamicTableFactoryTest {
             @Nullable Pattern topicPattern,
             Properties properties,
             StartupMode startupMode,
-            Map<KafkaTopicPartition, Long> specificStartupOffsets,
-            long startupTimestampMillis) {
+            Map<TopicPartition, Long> specificStartupOffsets,
+            long startupTimestampMillis,
+            @Nullable Integer parallelism) {
         return new KafkaDynamicSource(
                 physicalDataType,
                 keyDecodingFormat,
@@ -1267,7 +1355,8 @@ public class KafkaDynamicTableFactoryTest {
                 Collections.emptyMap(),
                 0,
                 false,
-                FactoryMocks.IDENTIFIER.asSummaryString());
+                FactoryMocks.IDENTIFIER.asSummaryString(),
+                parallelism);
     }
 
     private static KafkaDynamicSink createExpectedSink(
@@ -1280,10 +1369,11 @@ public class KafkaDynamicTableFactoryTest {
             @Nullable List<String> topics,
             @Nullable Pattern topicPattern,
             Properties properties,
-            @Nullable FlinkKafkaPartitioner<RowData> partitioner,
+            @Nullable KafkaPartitioner<RowData> partitioner,
             DeliveryGuarantee deliveryGuarantee,
             @Nullable Integer parallelism,
-            String transactionalIdPrefix) {
+            String transactionalIdPrefix,
+            TransactionNamingStrategy transactionNamingStrategy) {
         return new KafkaDynamicSink(
                 physicalDataType,
                 physicalDataType,
@@ -1300,7 +1390,8 @@ public class KafkaDynamicTableFactoryTest {
                 false,
                 SinkBufferFlushMode.DISABLED,
                 parallelism,
-                transactionalIdPrefix);
+                transactionalIdPrefix,
+                transactionNamingStrategy);
     }
 
     /**
