@@ -18,17 +18,19 @@
 
 package org.apache.flink.streaming.connectors.kafka.table;
 
-import org.apache.flink.api.common.restartstrategy.RestartStrategies;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.configuration.CoreOptions;
+import org.apache.flink.configuration.RestartStrategyOptions;
+import org.apache.flink.configuration.StateRecoveryOptions;
+import org.apache.flink.connector.base.DeliveryGuarantee;
+import org.apache.flink.connector.kafka.sink.KafkaPartitioner;
+import org.apache.flink.connector.kafka.sink.TransactionNamingStrategy;
 import org.apache.flink.core.execution.JobClient;
 import org.apache.flink.core.execution.SavepointFormatType;
-import org.apache.flink.runtime.jobgraph.SavepointConfigOptions;
 import org.apache.flink.runtime.messages.FlinkJobTerminatedWithoutCancellationException;
 import org.apache.flink.streaming.api.datastream.DataStream;
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
-import org.apache.flink.streaming.api.functions.sink.SinkFunction;
-import org.apache.flink.streaming.connectors.kafka.partitioner.FlinkKafkaPartitioner;
+import org.apache.flink.streaming.api.functions.sink.legacy.SinkFunction;
 import org.apache.flink.table.api.TableResult;
 import org.apache.flink.table.api.bridge.java.StreamTableEnvironment;
 import org.apache.flink.table.api.config.TableConfigOptions;
@@ -357,6 +359,55 @@ public class KafkaTableITCase extends KafkaTableTestBase {
         cleanupTopic(topic2);
     }
 
+    @Test
+    public void testExactlyOnceSink() throws Exception {
+        // we always use a different topic name for each parameterized topic,
+        // in order to make sure the topic can be created.
+        final String topic = "topics_" + format + "_" + UUID.randomUUID();
+        createTestTopic(topic, 1, 1);
+
+        // ---------- Produce an event time stream into Kafka -------------------
+        String bootstraps = getBootstrapServers();
+        tEnv.executeSql(
+                String.format(
+                        "CREATE TABLE sink (\n"
+                                + "  `user_id` INT,\n"
+                                + "  `item_id` INT,\n"
+                                + "  `behavior` STRING\n"
+                                + ") WITH (\n"
+                                + "  'connector' = '%s',\n"
+                                + "  'sink.delivery-guarantee' = '%s',\n"
+                                + "  'sink.transactional-id-prefix' = '%s',\n"
+                                + "  'sink.transaction-naming-strategy' = '%s',\n"
+                                + "  'topic' = '%s',\n"
+                                + "  'properties.bootstrap.servers' = '%s',\n"
+                                + "  'scan.startup.mode' = 'earliest-offset',\n"
+                                + "  'scan.bounded.mode' = 'latest-offset',\n"
+                                + "  %s\n"
+                                + ")\n",
+                        KafkaDynamicTableFactory.IDENTIFIER,
+                        DeliveryGuarantee.EXACTLY_ONCE,
+                        topic, // use topic as transactional id prefix - it's unique
+                        TransactionNamingStrategy.POOLING,
+                        topic,
+                        bootstraps,
+                        formatOptions()));
+
+        List<Row> values =
+                Arrays.asList(Row.of(1, 1102, "behavior 1"), Row.of(2, 1103, "behavior 2"));
+        tEnv.fromValues(values).insertInto("sink").execute().await();
+
+        // ---------- Consume stream from Kafka -------------------
+        List<Row> results = collectAllRows(tEnv.sqlQuery("SELECT * from sink"));
+        assertThat(results)
+                .containsExactlyInAnyOrder(
+                        Row.of(1, 1102, "behavior 1"), Row.of(2, 1103, "behavior 2"));
+
+        // ------------- cleanup -------------------
+
+        cleanupTopic(topic);
+    }
+
     @Test
     public void testKafkaSourceEmptyResultOnDeletedOffsets() throws Exception {
         // we always use a different topic name for each parameterized topic,
@@ -1251,12 +1302,12 @@ public class KafkaTableITCase extends KafkaTableTestBase {
         // ---------- Resume the consume job from savepoint  -------------------
 
         Configuration configuration = new Configuration();
-        configuration.set(SavepointConfigOptions.SAVEPOINT_PATH, savepointPath);
+        configuration.set(StateRecoveryOptions.SAVEPOINT_PATH, savepointPath);
+        configuration.set(RestartStrategyOptions.RESTART_STRATEGY, "disable");
         configuration.set(CoreOptions.DEFAULT_PARALLELISM, 1);
         StreamExecutionEnvironment env =
                 StreamExecutionEnvironment.getExecutionEnvironment(configuration);
         StreamTableEnvironment tEnv = StreamTableEnvironment.create(env);
-        env.getConfig().setRestartStrategy(RestartStrategies.noRestart());
 
         tEnv.executeSql(createTable);
         tEnv.executeSql(createSink);
@@ -1467,7 +1518,7 @@ public class KafkaTableITCase extends KafkaTableTestBase {
     // --------------------------------------------------------------------------------------------
 
     /** Extract the partition id from the row and set it on the record. */
-    public static class TestPartitioner extends FlinkKafkaPartitioner<RowData> {
+    public static class TestPartitioner implements KafkaPartitioner<RowData> {
 
         private static final long serialVersionUID = 1L;
         private static final int PARTITION_ID_FIELD_IN_SCHEMA = 0;
