@@ -25,6 +25,7 @@ import org.apache.flink.api.connector.source.Boundedness;
 import org.apache.flink.api.dag.Transformation;
 import org.apache.flink.connector.base.DeliveryGuarantee;
 import org.apache.flink.connector.kafka.sink.KafkaSink;
+import org.apache.flink.connector.kafka.sink.TransactionNamingStrategy;
 import org.apache.flink.connector.kafka.source.KafkaSource;
 import org.apache.flink.connector.kafka.source.KafkaSourceTestUtils;
 import org.apache.flink.connector.kafka.source.enumerator.KafkaSourceEnumState;
@@ -74,6 +75,8 @@ import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.ExpectedException;
 
+import javax.annotation.Nullable;
+
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
@@ -84,9 +87,10 @@ import java.util.Properties;
 import java.util.function.Consumer;
 import java.util.regex.Pattern;
 
-import static org.apache.flink.core.testutils.FlinkMatchers.containsCause;
+import static org.apache.flink.core.testutils.FlinkAssertions.anyCauseMatches;
 import static org.apache.flink.streaming.connectors.kafka.table.KafkaConnectorOptions.ScanBoundedMode;
 import static org.apache.flink.streaming.connectors.kafka.table.KafkaConnectorOptionsUtil.AVRO_CONFLUENT;
+import static org.apache.flink.table.factories.FactoryUtil.SOURCE_PARALLELISM;
 import static org.apache.flink.table.factories.utils.FactoryMocks.createTableSink;
 import static org.apache.flink.table.factories.utils.FactoryMocks.createTableSource;
 import static org.assertj.core.api.Assertions.assertThat;
@@ -168,7 +172,8 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
                         SOURCE_VALUE_FIELDS,
                         null,
                         Collections.singletonList(SOURCE_TOPIC),
-                        UPSERT_KAFKA_SOURCE_PROPERTIES);
+                        UPSERT_KAFKA_SOURCE_PROPERTIES,
+                        null);
         assertThat(actualSource).isEqualTo(expectedSource);
 
         final KafkaDynamicSource actualUpsertKafkaSource = (KafkaDynamicSource) actualSource;
@@ -177,6 +182,38 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
         assertKafkaSource(provider);
     }
 
+    @Test
+    public void testTableSourceWithParallelism() {
+        final DataType producedDataType = SOURCE_SCHEMA.toPhysicalRowDataType();
+        // Construct table source using options and table source factory
+        final Map<String, String> modifiedOptions =
+                getModifiedOptions(
+                        getFullSourceOptions(),
+                        options -> options.put(SOURCE_PARALLELISM.key(), "100"));
+        final DynamicTableSource actualSource = createTableSource(SOURCE_SCHEMA, modifiedOptions);
+
+        final KafkaDynamicSource expectedSource =
+                createExpectedScanSource(
+                        producedDataType,
+                        keyDecodingFormat,
+                        valueDecodingFormat,
+                        SOURCE_KEY_FIELDS,
+                        SOURCE_VALUE_FIELDS,
+                        null,
+                        Collections.singletonList(SOURCE_TOPIC),
+                        UPSERT_KAFKA_SOURCE_PROPERTIES,
+                        100);
+        assertThat(actualSource).isEqualTo(expectedSource);
+
+        final KafkaDynamicSource actualUpsertKafkaSource = (KafkaDynamicSource) actualSource;
+        ScanTableSource.ScanRuntimeProvider provider =
+                actualUpsertKafkaSource.getScanRuntimeProvider(ScanRuntimeProviderContext.INSTANCE);
+        assertThat(provider).isInstanceOf(DataStreamScanProvider.class);
+        final DataStreamScanProvider sourceProvider = (DataStreamScanProvider) provider;
+        assertThat(sourceProvider.getParallelism()).isPresent();
+        assertThat(sourceProvider.getParallelism()).hasValue(100);
+    }
+
     @Test
     public void testTableSourceWithTopicList() {
         final Map<String, String> modifiedOptions =
@@ -199,7 +236,8 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
                         SOURCE_VALUE_FIELDS,
                         null,
                         Arrays.asList(SOURCE_TOPIC, SOURCE_TOPIC),
-                        UPSERT_KAFKA_SOURCE_PROPERTIES);
+                        UPSERT_KAFKA_SOURCE_PROPERTIES,
+                        null);
         assertThat(actualSource).isEqualTo(expectedSource);
 
         final KafkaDynamicSource actualUpsertKafkaSource = (KafkaDynamicSource) actualSource;
@@ -234,7 +272,8 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
                         DeliveryGuarantee.EXACTLY_ONCE,
                         SinkBufferFlushMode.DISABLED,
                         null,
-                        "kafka-sink");
+                        "kafka-sink",
+                        TransactionNamingStrategy.DEFAULT);
 
         // Test sink format.
         final KafkaDynamicSink actualUpsertKafkaSink = (KafkaDynamicSink) actualSink;
@@ -276,7 +315,8 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
                         DeliveryGuarantee.EXACTLY_ONCE,
                         SinkBufferFlushMode.DISABLED,
                         null,
-                        "kafka-sink");
+                        "kafka-sink",
+                        TransactionNamingStrategy.DEFAULT);
 
         // Test sink format.
         final KafkaDynamicSink actualUpsertKafkaSink = (KafkaDynamicSink) actualSink;
@@ -321,7 +361,8 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
                         DeliveryGuarantee.EXACTLY_ONCE,
                         new SinkBufferFlushMode(100, 1000L),
                         null,
-                        "kafka-sink");
+                        "kafka-sink",
+                        TransactionNamingStrategy.DEFAULT);
 
         // Test sink format.
         final KafkaDynamicSink actualUpsertKafkaSink = (KafkaDynamicSink) actualSink;
@@ -376,7 +417,8 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
                         DeliveryGuarantee.EXACTLY_ONCE,
                         SinkBufferFlushMode.DISABLED,
                         100,
-                        "kafka-sink");
+                        "kafka-sink",
+                        TransactionNamingStrategy.DEFAULT);
         assertThat(actualSink).isEqualTo(expectedSink);
 
         final DynamicTableSink.SinkRuntimeProvider provider =
@@ -387,6 +429,41 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
         assertThat((long) sinkProvider.getParallelism().get()).isEqualTo(100);
     }
 
+    @Test
+    public void testTableSinkStrategyTranslation() {
+        for (TransactionNamingStrategy namingStrategy : TransactionNamingStrategy.values()) {
+            final EncodingFormat<SerializationSchema<RowData>> valueEncodingFormat =
+                    new TestFormatFactory.EncodingFormatMock(",");
+            final Map<String, String> modifiedOptions =
+                    getModifiedOptions(
+                            getFullSinkOptions(),
+                            options -> {
+                                options.put("sink.delivery-guarantee", "exactly-once");
+                                options.put("sink.transactional-id-prefix", "kafka-sink");
+                                options.put(
+                                        "sink.transaction-naming-strategy", namingStrategy.name());
+                            });
+            final DynamicTableSink actualSink = createTableSink(SINK_SCHEMA, modifiedOptions);
+            final DynamicTableSink expectedSink =
+                    createExpectedSink(
+                            SINK_SCHEMA.toPhysicalRowDataType(),
+                            keyEncodingFormat,
+                            valueEncodingFormat,
+                            SINK_KEY_FIELDS,
+                            SINK_VALUE_FIELDS,
+                            null,
+                            Collections.singletonList(SINK_TOPIC),
+                            null,
+                            UPSERT_KAFKA_SINK_PROPERTIES,
+                            DeliveryGuarantee.EXACTLY_ONCE,
+                            SinkBufferFlushMode.DISABLED,
+                            null,
+                            "kafka-sink",
+                            namingStrategy);
+            assertThat(actualSink).isEqualTo(expectedSink);
+        }
+    }
+
     @Test
     public void testTableSinkAutoCompleteSchemaRegistrySubject() {
         // value.format + key.format
@@ -631,116 +708,120 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
 
     @Test
     public void testCreateSourceTableWithoutPK() {
-        thrown.expect(ValidationException.class);
-        thrown.expect(
-                containsCause(
-                        new ValidationException(
-                                "'upsert-kafka' tables require to define a PRIMARY KEY constraint. "
-                                        + "The PRIMARY KEY specifies which columns should be read from or write to the Kafka message key. "
-                                        + "The PRIMARY KEY also defines records in the 'upsert-kafka' table should update or delete on which keys.")));
-
         ResolvedSchema illegalSchema =
                 ResolvedSchema.of(
                         Column.physical("window_start", DataTypes.STRING()),
                         Column.physical("region", DataTypes.STRING()),
                         Column.physical("view_count", DataTypes.BIGINT()));
-        createTableSource(illegalSchema, getFullSourceOptions());
+
+        assertThatThrownBy(() -> createTableSource(illegalSchema, getFullSourceOptions()))
+                .isInstanceOf(ValidationException.class)
+                .satisfies(
+                        anyCauseMatches(
+                                ValidationException.class,
+                                "'upsert-kafka' tables require to define a PRIMARY KEY constraint. "
+                                        + "The PRIMARY KEY specifies which columns should be read from or write to the Kafka message key. "
+                                        + "The PRIMARY KEY also defines records in the 'upsert-kafka' table should update or delete on which keys."));
     }
 
     @Test
     public void testCreateSinkTableWithoutPK() {
-        thrown.expect(ValidationException.class);
-        thrown.expect(
-                containsCause(
-                        new ValidationException(
-                                "'upsert-kafka' tables require to define a PRIMARY KEY constraint. "
-                                        + "The PRIMARY KEY specifies which columns should be read from or write to the Kafka message key. "
-                                        + "The PRIMARY KEY also defines records in the 'upsert-kafka' table should update or delete on which keys.")));
-
         ResolvedSchema illegalSchema =
                 ResolvedSchema.of(
                         Column.physical("region", DataTypes.STRING()),
                         Column.physical("view_count", DataTypes.BIGINT()));
-        createTableSink(illegalSchema, getFullSinkOptions());
+
+        assertThatThrownBy(() -> createTableSink(illegalSchema, getFullSinkOptions()))
+                .isInstanceOf(ValidationException.class)
+                .satisfies(
+                        anyCauseMatches(
+                                ValidationException.class,
+                                "'upsert-kafka' tables require to define a PRIMARY KEY constraint. "
+                                        + "The PRIMARY KEY specifies which columns should be read from or write to the Kafka message key. "
+                                        + "The PRIMARY KEY also defines records in the 'upsert-kafka' table should update or delete on which keys."));
     }
 
     @Test
     public void testSerWithCDCFormatAsValue() {
-        thrown.expect(ValidationException.class);
-        thrown.expect(
-                containsCause(
-                        new ValidationException(
+        assertThatThrownBy(
+                        () ->
+                                createTableSink(
+                                        SINK_SCHEMA,
+                                        getModifiedOptions(
+                                                getFullSinkOptions(),
+                                                options ->
+                                                        options.put(
+                                                                String.format(
+                                                                        "value.%s.%s",
+                                                                        TestFormatFactory
+                                                                                .IDENTIFIER,
+                                                                        TestFormatFactory
+                                                                                .CHANGELOG_MODE
+                                                                                .key()),
+                                                                "I;UA;UB;D"))))
+                .isInstanceOf(ValidationException.class)
+                .satisfies(
+                        anyCauseMatches(
+                                ValidationException.class,
                                 String.format(
                                         "'upsert-kafka' connector doesn't support '%s' as value format, "
                                                 + "because '%s' is not in insert-only mode.",
                                         TestFormatFactory.IDENTIFIER,
-                                        TestFormatFactory.IDENTIFIER))));
-
-        createTableSink(
-                SINK_SCHEMA,
-                getModifiedOptions(
-                        getFullSinkOptions(),
-                        options ->
-                                options.put(
-                                        String.format(
-                                                "value.%s.%s",
-                                                TestFormatFactory.IDENTIFIER,
-                                                TestFormatFactory.CHANGELOG_MODE.key()),
-                                        "I;UA;UB;D")));
+                                        TestFormatFactory.IDENTIFIER)));
     }
 
     @Test
     public void testDeserWithCDCFormatAsValue() {
-        thrown.expect(ValidationException.class);
-        thrown.expect(
-                containsCause(
-                        new ValidationException(
+        assertThatThrownBy(
+                        () ->
+                                createTableSource(
+                                        SOURCE_SCHEMA,
+                                        getModifiedOptions(
+                                                getFullSourceOptions(),
+                                                options ->
+                                                        options.put(
+                                                                String.format(
+                                                                        "value.%s.%s",
+                                                                        TestFormatFactory
+                                                                                .IDENTIFIER,
+                                                                        TestFormatFactory
+                                                                                .CHANGELOG_MODE
+                                                                                .key()),
+                                                                "I;UA;UB;D"))))
+                .isInstanceOf(ValidationException.class)
+                .satisfies(
+                        anyCauseMatches(
+                                ValidationException.class,
                                 String.format(
                                         "'upsert-kafka' connector doesn't support '%s' as value format, "
                                                 + "because '%s' is not in insert-only mode.",
                                         TestFormatFactory.IDENTIFIER,
-                                        TestFormatFactory.IDENTIFIER))));
-
-        createTableSource(
-                SOURCE_SCHEMA,
-                getModifiedOptions(
-                        getFullSourceOptions(),
-                        options ->
-                                options.put(
-                                        String.format(
-                                                "value.%s.%s",
-                                                TestFormatFactory.IDENTIFIER,
-                                                TestFormatFactory.CHANGELOG_MODE.key()),
-                                        "I;UA;UB;D")));
+                                        TestFormatFactory.IDENTIFIER)));
     }
 
     @Test
     public void testInvalidSinkBufferFlush() {
-        thrown.expect(ValidationException.class);
-        thrown.expect(
-                containsCause(
-                        new ValidationException(
+        assertThatThrownBy(
+                        () ->
+                                createTableSink(
+                                        SINK_SCHEMA,
+                                        getModifiedOptions(
+                                                getFullSinkOptions(),
+                                                options -> {
+                                                    options.put("sink.buffer-flush.max-rows", "0");
+                                                    options.put("sink.buffer-flush.interval", "1s");
+                                                })))
+                .isInstanceOf(ValidationException.class)
+                .satisfies(
+                        anyCauseMatches(
+                                ValidationException.class,
                                 "'sink.buffer-flush.max-rows' and 'sink.buffer-flush.interval' "
                                         + "must be set to be greater than zero together to enable"
-                                        + " sink buffer flushing.")));
-        createTableSink(
-                SINK_SCHEMA,
-                getModifiedOptions(
-                        getFullSinkOptions(),
-                        options -> {
-                            options.put("sink.buffer-flush.max-rows", "0");
-                            options.put("sink.buffer-flush.interval", "1s");
-                        }));
+                                        + " sink buffer flushing."));
     }
 
     @Test
     public void testExactlyOnceGuaranteeWithoutTransactionalIdPrefix() {
-        thrown.expect(ValidationException.class);
-        thrown.expect(
-                containsCause(
-                        new ValidationException(
-                                "sink.transactional-id-prefix must be specified when using DeliveryGuarantee.EXACTLY_ONCE.")));
-
         final Map<String, String> modifiedOptions =
                 getModifiedOptions(
                         getFullSinkOptions(),
@@ -750,7 +831,12 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
                                     KafkaConnectorOptions.DELIVERY_GUARANTEE.key(),
                                     DeliveryGuarantee.EXACTLY_ONCE.toString());
                         });
-        createTableSink(SINK_SCHEMA, modifiedOptions);
+        assertThatThrownBy(() -> createTableSink(SINK_SCHEMA, modifiedOptions))
+                .isInstanceOf(ValidationException.class)
+                .satisfies(
+                        anyCauseMatches(
+                                ValidationException.class,
+                                "sink.transactional-id-prefix must be specified when using DeliveryGuarantee.EXACTLY_ONCE."));
     }
 
     // --------------------------------------------------------------------------------------------
@@ -851,7 +937,8 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
             int[] valueFields,
             String keyPrefix,
             List<String> topic,
-            Properties properties) {
+            Properties properties,
+            @Nullable Integer parallelism) {
         return new KafkaDynamicSource(
                 producedDataType,
                 keyDecodingFormat,
@@ -869,7 +956,8 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
                 Collections.emptyMap(),
                 0,
                 true,
-                FactoryMocks.IDENTIFIER.asSummaryString());
+                FactoryMocks.IDENTIFIER.asSummaryString(),
+                parallelism);
     }
 
     private static KafkaDynamicSink createExpectedSink(
@@ -885,7 +973,8 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
             DeliveryGuarantee deliveryGuarantee,
             SinkBufferFlushMode flushMode,
             Integer parallelism,
-            String transactionalIdPrefix) {
+            String transactionalIdPrefix,
+            TransactionNamingStrategy transactionNamingStrategy) {
         return new KafkaDynamicSink(
                 consumedDataType,
                 consumedDataType,
@@ -902,7 +991,8 @@ public class UpsertKafkaDynamicTableFactoryTest extends TestLogger {
                 true,
                 flushMode,
                 parallelism,
-                transactionalIdPrefix);
+                transactionalIdPrefix,
+                transactionNamingStrategy);
     }
 
     private KafkaSource<?> assertKafkaSource(ScanTableSource.ScanRuntimeProvider provider) {
